      SUBROUTINE CALPBLREGIME(PBLREGIME)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .     
C SUBPROGRAM:    CALPBL COMPUTES PBL HEIGHT BASED ON BULK RCH NUMBER
C     
C ABSTRACT:  
C   THIS ROUTINE COMPUTES THE BULK RICHARDSON NUMBER BASED ON ALGORITHMS
C   FROM WRF SURFACE LAYER AND THEN DERIVE PBL REGIME AS FOLLOWS:
!        1. BR .GE. 0.2;
!               REPRESENTS NIGHTTIME STABLE CONDITIONS (REGIME=1),
!
!        2. BR .LT. 0.2 .AND. BR .GT. 0.0;
!               REPRESENTS DAMPED MECHANICAL TURBULENT CONDITIONS
!               (REGIME=2),
!
!        3. BR .EQ. 0.0
!               REPRESENTS FORCED CONVECTION CONDITIONS (REGIME=3),
!
!        4. BR .LT. 0.0
!               REPRESENTS FREE CONVECTION CONDITIONS (REGIME=4).    
C   .     
C     
C PROGRAM HISTORY LOG:
C   07-04-27  H CHUANG 
C   
C USAGE:    CALL CALPBLREGIME(PBLREGIME)
C   INPUT ARGUMENT LIST:
C
C   OUTPUT ARGUMENT LIST: 
C     PBLRI  - PBL HEIGHT ABOVE GROUND
C     
C   OUTPUT FILES:
C     NONE
C     
C   SUBPROGRAMS CALLED:
C     UTILITIES:
C       NONE
C     LIBRARY:
C       COMMON   - 
C                  CTLBLK
C     
C   ATTRIBUTES:
C     LANGUAGE: FORTRAN
C     MACHINE : 
C$$$  
C
      use vrbls3d
      use vrbls2d
      use masks
C     
C     INCLUDE,DERIVE,SET PARAMETERS.
C     
!      INCLUDE "parmeta"
!      PARAMETER(LM1=LM-1)
      INCLUDE "params"
      REAL    , PARAMETER ::  VCONVC=1.
C
C     INCLUDE COMMON BLOCKS.
C
      INCLUDE "CTLBLK.comm"
      INCLUDE "GRIDSPEC.comm"
C     
C     DECLARE VARIABLES.
C     
      
      REAL PBLREGIME(IM,jsta_2l:jend_2u)
     
C
C     
C*************************************************************************
C     
C     INITIALIZE ARRAYS.
C
!$omp  parallel do
        DO J=JSTA,JEND
        DO I=1,IM
          PBLREGIME(I,J)=SPVAL
        ENDDO
        ENDDO
C
C     COMPUTE BULK RICHARDSON NUMBER AS CODED IN WRF module_sf_sfclay
C
!$omp  parallel do
!$omp& private(uhkl,ulkl,vhkl,vlkl,rib,ubot,utop,vbot,vtop,
!$omp&         betta,ricr,ustarr,wmin,tvhtop,ztop,
!$omp&         wndsl,wndslp,betta,ricr,ustarr,wmin 
!$omp&       ,IFRSTLEV
!$omp&       ,ICALPBL
!$omp&       ,LVLP
!$omp&       ,RIF
!$omp&       ,RIBP
!$omp&       ,UBOT1
!$omp&       ,VBOT1
!$omp&       ,ZBOT1
!$omp&       ,THVBOT1)
C
      IF(MODELNAME .EQ. 'NMM')THEN
	 call exch(UH(1,jsta_2l,LM))
         call exch(VH(1,jsta_2l,LM))
      END IF
      	 	         
      DO J=JSTA_M,JEND_M
      DO I=2,IM-1
C
        APE=(P1000/PMID(I,J,LM))**CAPA
	THX=T(I,J,LM)*APE
	THVX=(Q(I,J,LM)*D608+H1)*THX
        GOVRTH=G/THX
	IF(MODELNAME .EQ. 'NMM')THEN
	 IE=I+MOD(J+1,2) 
         IW=I+MOD(J+1,2)-1
	 UMASS=(UH(I,J-1,LM)+UH(IW,J,LM)+UH(IE,J,LM)
     +	       +UH(I,J+1,LM))/4.0
         VMASS=(VH(I,J-1,LM)+VH(IW,J,LM)+VH(IE,J,LM)
     +	       +VH(I,J+1,LM))/4.0
	 WSPD= SQRT(UMASS*UMASS+
     +	            VMASS*VMASS) 
	ELSE
         WSPD=SQRT(UH(I,J,LM)*UH(I,J,LM)+
     +	            VH(I,J,LM)*VH(I,J,LM))
        END IF
                                                                                 
        TSKV=THS(I,J)*(1.+D608*QS(I,J)*SMSTAV(I,J))
        DTHV=(THVX-TSKV)
!  Convective velocity scale Vc and subgrid-scale velocity Vsg
!  following Beljaars (1995, QJRMS) and Mahrt and Sun (1995, MWR)
!                                ... HONG Aug. 2001
!
        rhox=PINT(I,J,LM+1)/RD/(T(I,J,LM)
     &  	*(Q(I,J,LM)*D608+H1)) !density
        fluxc = max(-twbs(i,j)/rhox/cp                    
     &              - d608*tskv*QWBS(i,j)/rhox,0.)
        tsfc=THS(I,J)*(PINT(I,J,LM+1)/P1000)**CAPA
        VCONV = vconvc*(g/tsfc*pblh(i,j)*fluxc)**.33
! VCONV comes from Beljaars only
        VSGD = 0.32 * (max(dx(i,j)/5000.-1.,0.))**.33
        WSPD=SQRT(WSPD*WSPD+VCONV*VCONV+vsgd*vsgd)
        WSPD=AMAX1(WSPD,0.1)
        BR=GOVRTH*(ZMID(I,J,LM)-ZINT(I,J,LM+1))*DTHV
     &	  /(WSPD*WSPD)
     
	IF(BR < 0.0)THEN
	 PBLREGIME(I,J)=4.0
	ELSE IF(BR == 0.0)THEN
	 PBLREGIME(I,J)=3.0
	ELSE IF(BR < 0.2)THEN
	 PBLREGIME(I,J)=2.0
	ELSE
	 PBLREGIME(I,J)=1.0 
	END IF
	
	ii=im/2
	jj=(jsta+jend)/2
	if(i==ii.and.j==jj)print*,'Debug: CALPBLREGIME ',i,j,br,
     +    PBLREGIME(I,J)	
	   
      ENDDO
      ENDDO
!      
C     END OF ROUTINE.
C     
      RETURN
      END

